--[[
   *** Tank Warnings ***
   
   Warning messages for tanks heavily inspired in Tankalyze
   
   mumbles Turalyon EU rui@ngen.org
   http://www.wowace.com/projects/tankwarnings
   svn://svn.wowace.com/wow/tankwarnings/mainline/trunk

   version 0.21
   $Revision$
]]--

-- Addon Init
TankWarnings = LibStub("AceAddon-3.0"):NewAddon("TankWarnings", "AceConsole-3.0", "AceEvent-3.0", "LibSink-2.0")
local tw = TankWarnings
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local db
local registry = LibStub("AceConfigRegistry-3.0")
local dialog = LibStub("AceConfigDialog-3.0")

-- Locale
local AceLocale = LibStub("AceLocale-3.0")
local L = AceLocale:GetLocale("TankWarnings")

-- Our class
_, tw.englishClass = UnitClass("player")

--[[ 
    Table with class spell ids to easily reference later the messages and
    settings on user options
]]--
tw.spellids = {
    ["WARRIOR"] = {
        [355] = "taunt",
        [1161] = "shout",
        [694] = "mocking",
        [871] = "shieldwall",
        [12975] = "laststand",
    },
    ["DRUID"] = {
        [6795] = "growl",
        [5209] = "roar",
    },
    ["PALADIN"] = {
        [31789] = "righteous",
        [498] = "divine",
        [1022] = "hop",
        [5599] = "hop",
        [10278] = "hop",
    },
    ["DEATHKNIGHT"] = {
        [56222] = "darkcomman",
        [48792] = "ibf",
        [42650] = "aotd",
        [42651] = "aotd",
        [48707] = "ams",
        [51052] = "amz",
    }
}

--[[
    OnInitialize
    
    Function called by Ace3 uppon loading
    
    @return void
]]--
function tw:OnInitialize()
    db = AceDB:New("TankWarningsDB", self.defaults)
    self.db = db
    self.options.args.profile = AceDBOptions:GetOptionsTable(db)

    self:SetSinkStorage(self.db.profile.messages.sinkOptions)
   
    registry:RegisterOptionsTable("TankWarnings", tw.options)
    self:RegisterChatCommand("tw", function() dialog:Open("TankWarnings") end)
    self:RegisterChatCommand("tankwarnings", function() dialog:Open("TankWarnings") end)
    dialog:AddToBlizOptions("TankWarnings", "TankWarnings")
end

--[[
    OnEnable
    
    Function called by Ace3 when the addon is Enabled by the used
    
    @return void
]]--
function tw:OnEnable()
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED") -- Taunt, Growl & Mocking should stay in here
end

--[[
    OnDisable
    
    Function called by Ace3 when the addon is Disabled by the used
    
    @return void
]]--
function tw:OnDisable()
end

--[[
    COMBAT_LOG_EVENT_UNFILTERED
    
    Function triggered when the evenet witht he same name is triggered to parse resists and announcements
    Read http://www.wowwiki.com/API_COMBAT_LOG_EVENT for more info
    
    @params string  event
    @params integer timestamp
    @params string  subevent
    @params integer srcGUID
    @params string  srcname
    @params integer srcflags
    @params integer dstGUID
    @params string  dstname
    @params integer dstflags
    @params integer spellID
    @params string  spellname
    @params integer spellschool
    @return void
]]--
function tw:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, subevent, srcGUID, srcname, srcflags, dstGUID, dstname, dstflags, spellID, spellname, spellschool)
    -- If we should be disabled for PvP (Bg's or arenas) no point in continuing after this
    if self.db.profile.messages.disableinpvp and self:IsInPvP() then
        return
    end

    if UnitGUID("player") == srcGUID then
        -- Resists
        if subevent == "SPELL_MISSED" then
            local level = UnitLevel("target")
            local guid = UnitGUID("target")
            if guid ~= dstGUID then level = nil end
            self:Announce(self:GetResistMessage(spellID), dstname, level)
        end
        
        -- Announcements
        if subevent == "SPELL_CAST_SUCCESS" then
            if BigWigs and db.profile.announcements.bigwigs.enable then self:ProcessBarTimers(spellID) end
            self:Announce(self:GetAnnounceMessage(spellID), nil, nil)
        end
    end
end

--[[
    GetResistMessage
    
    Returns the message specified by the user in the config that should be displayer for the given spell resist
    
    @params integer spellid The spell id of the spell used
    @return string  The message configured to be displayed for the given spell resist
]]--
function tw:GetResistMessage(spellid)
    if self.spellids[self.englishClass][spellid] ~= nil then
        dbval = self.spellids[self.englishClass][spellid]
        return self.db.profile.messages.resists[dbval]
    end
end

--[[
    GetAnnounceMessage
    
    Returns the message specified by the user in the config that should be displayed for the given spell announcement
    
    @params integer spellid The spell id of the spell used
    @return string  The message configured to be displayed for the given spell announce
]]--
function tw:GetAnnounceMessage(spellid)
    if self.spellids[self.englishClass][spellid] ~= nil then
        dbval = self.spellids[self.englishClass][spellid]
        return self.db.profile.messages.announcements[dbval]
    end
end

--[[
    Announce
    
    Announces through LibSink and/or Raid warning depending on user preferences the specified message
    
    @params string  message The message to be displayed, required
    @params string  target  Target name on which the ability was used or (in case of AoE abilities like aoe shouts) the one who got the resist
    @params integer level   The level of the target
    return void
]]--
function tw:Announce(message, target, level)
    if level == -1 then
        level = L["Boss"]
    end

    msg = nil
    if target ~= nil and level ~= nil then
        msg = string.gsub(string.gsub(message, "{t}", target), "{l}",  level)
    elseif target ~= nil then
        msg = string.gsub(message, "{t}", target)
    else
        msg = message
    end
    
    if msg == nil then return end

    -- Message output
    if self.db.profile.messages.sendraid then
        -- First check if our sink is sending to raid already, if it is, bail out no need to send this
        if self.db.profile.messages.sinkOptions.sink20OutputSink ~= "RaidWarning" then
            -- Check if were in a raid, if we are, check if we have permissions to send the message there
            if GetNumRaidMembers() > 0 then
                if self:IsPromoted() then
                    SendChatMessage(msg, "RAID_WARNING")
                else
                    self:Print(L["Cannot send message to raid, not promoted"])
                end
            elseif GetNumPartyMembers() > 0 then -- in a party ? send to raid warning anyway, doesn't need promotion
                SendChatMessage(msg, "RAID_WARNING")
            end
        end
    end

    -- send our message through the sink
    self:Pour(msg, 1, 0, 0, nil, 24, "OUTLINE", true)
end

--[[
    ProcessBarTimers
    
    Function to create timer bars using 3rd party addons like BigWigs displaying the time left on abilities used in announcements
    like Shield Wall, Last Stand, Challenging Shout etc
    For now, only supports BigWigs and requires Custom Bars module enabled (part of BigWigs_Extras addon)
    
    @param integer skillid The skill id used in the announcement
    @return void
]]--
function tw:ProcessBarTimers(skillid)
    -- If the spellid is not one of our announcements to make and enabled, bail out
    if self.spellids[self.englishClass][skillid] == nil then return end
    
    dbval = self.spellids[self.englishClass][skillid]
    if not self.db.profile.announcements[dbval] then return end

    -- Get skill info
    local name = GetSpellInfo(skillid)
    -- Find our buff to query it's duration to display the bar
    local timeleft
    for i=1, 40 do
        local buff, _, _, _, _, tleft = UnitBuff("player", i, true)
        if buff == name then
            timeleft = tleft
        end
    end

    -- something went wrong we can't find our buff....
    if timeleft == nil then return end
    
    -- does the buff last time enough for us to call a timer bar ?
    if timeleft < db.profile.announcements.bigwigs.mintime then return end
    
    -- Does the module exists ?
    if not BigWigs:HasModule("Custom Bars") then
        -- warn the user it doesn't exist
        self:Print(L["BigWigs not enabled or Custom Bars module not loaded (part of BigWigs_Extras)"])
        return 
    end

    -- Get the module
    module = BigWigs:GetModule("Custom Bars")
    message = string.format("%s %s - %s", timeleft, UnitName("player"), name)
    if db.profile.announcements.bigwigs.global then
        module:TriggerEvent("BigWigs_SendSync", "BWCustomBar "..message)
    else
        module:StartBar(message, nil, true)
    end
end

--[[
    Test
    
    Function called to test the announcement of messages from the configuration UI
    
    @return void
]]--
function tw:Test()
    self:Announce(L["Resist Test Message"], UnitName("player"), UnitLevel("player"))
    self:Announce(L["Announce Test Message"], UnitName("player"), UnitLevel("player"))
end

--[[
    IsPromoted
    
    Verifies if the user is raid leader or promoted to assistance in the raid
    
    @return boolean true if promoted or leader false otherwise
]]--
function tw:IsPromoted()
	return IsRaidLeader() or IsRaidOfficer() and true or nil
end

--[[
    IsInPvP
    
    Checks if the user is in Battleground or Arenas
    
    @return boolean true if in a bg or arena, false otherwise
]]--
function tw:IsInPvP()
    return UnitIsPVPFreeForAll("player")
end

--[[
    FilterOptionsByClass
    
    Checks if the paramater option should be shown to the user or not depending on his class
    
    @params string class The class that the parameter should show to
    @return boolean Returns true if the class is not the same as the user class, false otherwise
]]--
function tw:FilterOptionsByClass(class)
    if tw.englishClass == class then
        return false
    end
    
    return true
end

--[[
    Configuration Options Table
]]--
tw.options = {
    type = "group",
    name = L["Tank Warnings"],
    args = {
        resists = {
            type = "group",
            name = L["Resists"],
            desc = L["Settings for resists"],
            order = 10,
            args = {
                taunt = {
                    type = "toggle",
                    name = L["Taunt"],
                    desc = L["Announces resisted taunts"],
                    get = function(info) return db.profile.resists.taunt end,
                    set = function(info, v) db.profile.resists.taunt = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 1,
                },
                growl = {
                    type = "toggle",
                    name = L["Growl"],
                    desc = L["Announces resisted growls"],
                    get = function(info) return db.profile.resists.growl end,
                    set = function(info, v) db.profile.resists.growl = v end,
                    hidden = tw:FilterOptionsByClass("DRUID"),
                    width = "full",
                    order = 2,
                },
                mocking = {
                    type = "toggle",
                    name = L["Mocking Blow"],
                    desc = L["Announces missed Mocking Blows"],
                    get = function(info) return db.profile.resists.mocking end,
                    set = function(info, v) db.profile.resists.mocking = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 3,
                },
                shout = {
                    type = "toggle",
                    name = L["Challenging Shout"],
                    desc = L["Announces resisted challenging shouts"],
                    get = function(info) return db.profile.resists.shout end,
                    set = function(info, v) db.profile.resists.shout = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 4,
                },
                roar = {
                    type = "toggle",
                    name = L["Challenging Roar"],
                    desc = L["Announces resisted challenging roars"],
                    get = function(info) return db.profile.resists.roar end,
                    set = function(info, v) db.profile.resists.roar = v end,
                    hidden = tw:FilterOptionsByClass("DRUID"),
                    width = "full",
                    order = 5,
                },
                righteous = {
                    type = "toggle",
                    name = L["Righteous Defense"],
                    desc = L["Announces resisted righteous defense"],
                    get = function(info) return db.profile.resists.roar end,
                    set = function(info, v) db.profile.resists.roar = v end,
                    hidden = tw:FilterOptionsByClass("PALADIN"),
                    width = "full",
                    order = 6,
                },
                darkcommand = {
                    type = "toggle",
                    name = L["Dark Command"],
                    desc = L["Announces resisted dark command"],
                    get = function(info) return db.profile.resists.roar end,
                    set = function(info, v) db.profile.resists.roar = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 7,
                },
            },
        },
        announcements = {
            type = "group",
            name = L["Announcements"],
            desc = L["Settings for announcements"],
            order = 20,
            args = {
                shieldwall = {
                    type = "toggle",
                    name = L["Shield Wall"],
                    desc = L["Shield Wall"],
                    get = function(info) return db.profile.announcements.shieldwall end,
                    set = function(info, v) db.profile.announcements.shieldwall = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 1,
                },
                laststand = {
                    type = "toggle",
                    name = L["Last Stand"],
                    desc = L["Last Stand"],
                    get = function(info) return db.profile.announcements.laststand end,
                    set = function(info, v) db.profile.announcements.laststand = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 2,
                },
                shout = {
                    type = "toggle",
                    name = L["Challenging Shout"],
                    desc = L["Challenging Shout"],
                    get = function(info) return db.profile.announcements.shout end,
                    set = function(info, v) db.profile.announcements.shout = v end,
                    hidden = tw:FilterOptionsByClass("WARRIOR"),
                    width = "full",
                    order = 3,
                },
                roar = {
                    type = "toggle",
                    name = L["Challenging Roar"],
                    desc = L["Challenging Roar"],
                    get = function(info) return db.profile.announcements.roar end,
                    set = function(info, v) db.profile.announcements.roar = v end,
                    hidden = tw:FilterOptionsByClass("DRUID"),
                    width = "full",
                    order = 4,
                },
                divine = {
                    type = "toggle",
                    name = L["Divine Protection"],
                    desc = L["Divine Protection"],
                    get = function(info) return db.profile.announcements.divine end,
                    set = function(info, v) db.profile.announcements.divine = v end,
                    hidden = tw:FilterOptionsByClass("PALADIN"),
                    width = "full",
                    order = 5,
                },
                hop = {
                    type = "toggle",
                    name = L["Hand of Protection"],
                    desc = L["Hand of Protection"],
                    get = function(info) return db.profile.announcements.hop end,
                    set = function(info, v) db.profile.announcements.hop = v end,
                    hidden = tw:FilterOptionsByClass("PALADIN"),
                    width = "full",
                    order = 6,
                },
                ibf = {
                    type = "toggle",
                    name = L["Icebound Fortitude"],
                    desc = L["Icebound Fortitude"],
                    get = function(info) return db.profile.announcements.ibf end,
                    set = function(info, v) db.profile.announcements.ibf = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 7,
                },
                aotd = {
                    type = "toggle",
                    name = L["Army of the Dead"],
                    desc = L["Army of the Dead"],
                    get = function(info) return db.profile.announcements.aotd end,
                    set = function(info, v) db.profile.announcements.aotd = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 8,
                },
                ams = {
                    type = "toggle",
                    name = L["Anti-Magic Shell"],
                    desc = L["Anti-Magic Shell"],
                    get = function(info) return db.profile.announcements.ams end,
                    set = function(info, v) db.profile.announcements.ams = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 8,
                },
                amz = {
                    type = "toggle",
                    name = L["Anti-Magic Zone"],
                    desc = L["Anti-Magic Zone"],
                    get = function(info) return db.profile.announcements.amz end,
                    set = function(info, v) db.profile.announcements.amz = v end,
                    hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                    width = "full",
                    order = 9,
                },
                header = {
                    type = "header",
                    name = L["BigWigs Timers"],
                    width = "full",
                    order = 30,
                    hidden = function() if BigWigs then return false else return true end end,
                },
                enablebw = {
                    type = "toggle",
                    name = L["Enable BigWigs Timer Bars"],
                    desc = L["Enable the display of BigWigs timer bars of your skills showing the time they last"],
                    get = function(info) return db.profile.announcements.bigwigs.enable end,
                    set = function(info, v) db.profile.announcements.bigwigs.enable = v end,
                    order = 31,
                    width = "full",
                    hidden = function() if BigWigs then return false else return true end end,
                },
                global = {
                    type = "toggle",
                    name = L["Global Timer"],
                    desc = L["Make the timer global, showing for everyone in the raid instead of local only showing to you"],
                    get = function(info) return db.profile.announcements.bigwigs.global end,
                    set = function(info, v) db.profile.announcements.bigwigs.global = v end,
                    order = 32,
                    width = "full",
                    hidden = function() if BigWigs then return false else return true end end,
                },
                mintime = {
                    type = "range",
                    name = L["Minimum Skill Time"],
                    desc = L["Skills that last less than the minimum skill time will not show a timer bar, setting to 0 will always show"],
                    get = function(info) return db.profile.announcements.bigwigs.mintime end,
                    set = function(info,v) db.profile.announcements.bigwigs.mintime = v end,
                    order = 33,
                    width = "full",
                    min = 0,
                    max = 30,
                    step = 1,
                    hidden = function() if BigWigs then return false else return true end end,
                },
            },
        },
        messages = {
            type = "group",
            name = L["Messages"],
            args = {
                test = {
                    type = "execute",
                    name = L["Test"],
                    desc = L["Send test messages with current settings"],
                    func = function() tw:Test() end,
                    order = 1,
                    width = "full"
                },
                disableinpvp = {
                    type = "toggle",
                    name = L["Disable in PvP"],
                    desc = L["Disable when player is in Battlegrounds/Arenas"],
                    get = function(info) return db.profile.messages.disableinpvp end,
                    set = function(info, v) db.profile.message.disableinpvp = v end,
                    order = 2,
                    width = "full",
                },
                sendraid = {
                    type = "toggle",
                    name = L["Send messages to raid warning regardless"],
                    desc = L["Displays the messages in raid warning regardles of your output config, usefull if you choose to output to somewhere else that is not Raid Warning"],
                    get = function(info) return db.profile.messages.sendraid end,
                    set = function(info, v) db.profile.messages.sendraid = v end,
                    order = 3,
                    width = "full",
                },
                output = tw:GetSinkAce3OptionsDataTable(),
                resists = {
                    type = "group",
                    name = L["Resists Messages"],
                    desc = L["Message settings for resists"],
                    order = 10,
                    args = {
                        taunt = {
                            type = "input",
                            name = L["Taunt"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.taunt end,
                            set = function(info, v) db.profile.messages.resists.taunt = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 1,
                        },
                        growl = {
                            type = "input",
                            name = L["Growl"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.growl end,
                            set = function(info, v) db.profile.messages.resists.growl = v end,
                            hidden = tw:FilterOptionsByClass("DRUID"),
                            width = "full",
                            order = 2,
                        },
                        mocking = {
                            type = "input",
                            name = L["Mocking Blow"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.mocking end,
                            set = function(info, v) db.profile.messages.resists.mocking = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 3,
                        },
                        shout = {
                            type = "input",
                            name = L["Challenging Shout"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.shout end,
                            set = function(info, v) db.profile.messages.resists.shout = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 4,
                        },
                        roar = {
                            type = "input",
                            name = L["Challenging Roar"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.roar end,
                            set = function(info, v) db.profile.messages.resists.roar = v end,
                            hidden = tw:FilterOptionsByClass("DRUID"),
                            width = "full",
                            order = 5,
                        },
                        righteous = {
                            type = "input",
                            name = L["Righteous Defense"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.righteous end,
                            set = function(info, v) db.profile.messages.resists.righteous = v end,
                            hidden = tw:FilterOptionsByClass("PALADIN"),
                            width = "full",
                            order = 6,
                        },
                        darkcommand = {
                            type = "input",
                            name = L["Dark Command"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.resists.darkcommand end,
                            set = function(info, v) db.profile.messages.resists.darkcommand = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 7,
                        },
                    },
                },
                announcements = {
                    type = "group",
                    name = L["Announcement Messages"],
                    desc = L["Message settings for announcements"],
                    order = 10,
                    args = {
                        shieldwall = {
                            type = "input",
                            name = L["Shield Wall"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.shieldwall end,
                            set = function(info, v) db.profile.messages.announcements.shieldwall = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 1,
                        },
                        laststand = {
                            type = "input",
                            name = L["Last Stand"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.laststand end,
                            set = function(info, v) db.profile.messages.announcements.laststand = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 2,
                        },
                        shout = {
                            type = "input",
                            name = L["Challenging Shout"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.shout end,
                            set = function(info, v) db.profile.messages.announcements.shout = v end,
                            hidden = tw:FilterOptionsByClass("WARRIOR"),
                            width = "full",
                            order = 3,
                        },
                        roar = {
                            type = "input",
                            name = L["Challenging Roar"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.roar end,
                            set = function(info, v) db.profile.messages.announcements.roar = v end,
                            hidden = tw:FilterOptionsByClass("DRUID"),
                            width = "full",
                            order = 4,
                        },
                        divine = {
                            type = "input",
                            name = L["Divine Protection"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.divine end,
                            set = function(info, v) db.profile.messages.announcements.divine = v end,
                            hidden = tw:FilterOptionsByClass("PALADIN"),
                            width = "full",
                            order = 5,
                        },
                        hop = {
                            type = "input",
                            name = L["Hand of Protection"],
                            desc = L["MessagesInfo"],
                            get = function(info) return db.profile.messages.announcements.hop end,
                            set = function(info, v) db.profile.messages.announcements.hop = v end,
                            hidden = tw:FilterOptionsByClass("PALADIN"),
                            width = "full",
                            order = 6,
                        },
                        ibf = {
                            type = "input",
                            name = L["Icebound Fortitude"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.ibf end,
                            set = function(info, v) db.profile.messages.announcements.ibf = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 7,
                        },
                        aotd = {
                            type = "input",
                            name = L["Army of the Dead"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.aotd end,
                            set = function(info, v) db.profile.messages.announcements.aotd = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 8,
                        },
                        ams = {
                            type = "input",
                            name = L["Anti-Magic Shell"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.ams end,
                            set = function(info, v) db.profile.messages.announcements.ams = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 9,
                        },
                        amz = {
                            type = "input",
                            name = L["Anti-Magic Zone"],
                            desc = L["MessagesInfoNoTarget"],
                            get = function(info) return db.profile.messages.announcements.amz end,
                            set = function(info, v) db.profile.messages.announcements.amz = v end,
                            hidden = tw:FilterOptionsByClass("DEATHKNIGHT"),
                            width = "full",
                            order = 10,
                        },
                    },
                },
            },
        },
    },
}

--[[
    Default Values
]]--
tw.defaults = {
    profile = {
        resists = {
            taunt = true,
            growl = true,
            mocking = true,
            shout = true,
            roar = true,
            righteous = true,
            darkcommand = true,
            messages = {
            }
        },
        announcements = {
            shieldwall = true,
            laststand = true,
            shout = true,
            roar = true,
            divine = true,
            hop = true,
            ibf = true,
            aotd = true,
            ams = true,
            amz = true,
            bigwigs = {
                enable = true,
                global = true,
                mintime = 5,
            },
        },
        messages = {
            disableinpvp = true,
            sendraid = true,
            sinkOptions = {},
            resists = {
                taunt = L["TauntMessage"],
                growl = L["GrowlMessage"],
                mocking = L["MockingMessage"],
                shout = L["ShoutMessage"],
                roar = L["RoarMessage"],
                righteous = L["RighteousMessage"],
                darkcommand = L["DarkCommandMessage"],
            },
            announcements = {
                shieldwall = L["ShieldWallMessage"],
                laststand = L["LastStandMessage"],
                shout = L["ChallengingShoutMessage"],
                roar = L["ChallengingRoarMessage"],
                divine = L["DivineProtectionMessage"],
                hop = L["HandOfProtectionMessage"],
                ibf = L["IceboundFortitudeMessage"],
                aotd = L["ArmyOfTheDeadMessage"],
                ams = L["AntiMagicShellMessage"],
                amz = L["AntiMagicZoneMessage"],
            },
        },
    },
}
